function [res] = AVR_bw_mc(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  Subfunction of AVR_bw.m function for ADC bandwidth evaluation by means of
%% amplitude spectrum comparison with reference signal sampled by wideband DSO
%% directly at AVR ADC input pin.
%%  This function does the main calculation of ADC frequency transfer. It 
%% includes variation of input parameters used for Monte-Carlo uncertainty
%% analysis. 
%%
%% Version 2013.07.16 by Stanislav Maslan, s.maslan@seznam.cz
%% http://elektronika.kvalitne.cz/ATMEL/necoteorie/AVR_ETS_bandwidth/AVR_ETS_bandwidth.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  %% resample DSO wave
  % fix DSO time scale + MC noise
  m = length(par.t);
  tn = par.t*(par.tsr + par.uc_d_tstab*randn(1,1)) + par.uc_d_jt*randn(1,m);
  % stay within valid time range
  tn(find(tn<par.t(1))) = par.t(1);
  id = find(tn>par.t(end),1,'first');
  if(isempty(id))
    id = m;
  end
  % resample DSO wave to new time scale
  dso = interp1q(par.t,par.csv,tn(1:id));
    
  %% periodize DSO waveform
  m = floor(length(dso)/(par.n/par.avr_pn*par.dso_sr/par.avr_sr))*(par.n/par.avr_pn*par.dso_sr/par.avr_sr);
  dso = dso(1:m);
  
  %% MC apply DSO ADC nonlinearity
  % original transfer (linear)
  dso_tf = [min(dso) max(dso)];
  dso_tf = dso_tf + [-1 +1]*diff(dso_tf)*0.01;
  % generate uc_d_inl_p random x points in x scale
  dso_tf_rx = ([0 ([1/(par.uc_d_inl_p-1):1/(par.uc_d_inl_p-1):1-1/(par.uc_d_inl_p-1)] + (rand(1,par.uc_d_inl_p-2)-0.5)/par.uc_d_inl_p) 1])*diff(dso_tf)+dso_tf(1);
  % and the same in y scale
  dso_tf_ry = interp1(dso_tf,dso_tf,dso_tf_rx) + par.uc_d_inl*(dso_tf(end)-dso_tf(1))*randn(1,length(dso_tf_rx));
  % interpolate [dso_tf_rx,dso_tf_ry] points (our non-linear transfer)
  dso_tf_rxp = dso_tf(1):diff(dso_tf)/1024:dso_tf(end);
  dso_tf_ryp = interp1(dso_tf_rx,dso_tf_ry,dso_tf_rxp,'spline');
  % and apply it to data
  dso = interp1q(dso_tf_rxp',dso_tf_ryp',dso);
  
  %% MC apply par.avr ADC nonlinearity
  % original transfer (linear)
  avr_tf = [min(par.avr) max(par.avr)];
  avr_tf = avr_tf + [-1 +1]*diff(avr_tf)*0.01;
  % generate uc_d_inl_p random x points in x scale
  avr_tf_rx = ([0 ([1/(par.uc_a_inl_p-1):1/(par.uc_a_inl_p-1):1-1/(par.uc_a_inl_p-1)] + (rand(1,par.uc_a_inl_p-2)-0.5)/par.uc_a_inl_p) 1])*diff(avr_tf)+avr_tf(1);
  % and the same in y scale
  avr_tf_ry = interp1(avr_tf,avr_tf,avr_tf_rx) + par.uc_a_inl*diff(avr_tf)*randn(1,length(avr_tf_rx));
  % interpolate [avr_tf_rx,avr_tf_ry] points (our non-linear transfer)
  avr_tf_rxp = avr_tf(1):diff(avr_tf)/1024:avr_tf(end);
  avr_tf_ryp = interp1(avr_tf_rx,avr_tf_ry,avr_tf_rxp,'spline');
  % and apply it to data
  m_avr = interp1q(avr_tf_rxp',avr_tf_ryp',par.avr);
      
  %% add MC noise (sampling quantization noise)
  m_dso = dso + dso.*randn(m,1)*par.uc_d_noise;
  m_avr = m_avr + par.avr.*randn(par.n,1)*par.uc_a_noise;

  %% get DSO spectrum
  % full spectrum
  dso_f = linspace(0,par.dso_sr*0.5,m/2);
  dso_s = fft(m_dso);
  dso_s = dso_s(1:floor(m/2));
  
  % select odd harmonics up to par.avr nyquist limit
  dso_k = round(m/2*par.avr_sr/par.dso_sr);
  dso_ks = round(par.sig_f/par.dso_sr*m);
  dso_k = dso_k - dso_ks;
  dso_f = dso_f(dso_ks+1:2*dso_ks:dso_k);
  dso_s = dso_s(dso_ks+1:2*dso_ks:dso_k);
  % amplitude and phase
  dso_fi = arg(dso_s);
  dso_s = abs(dso_s);
  % add MC noise (gain flatness error)
  dso_s = dso_s + dso_s.*randn(length(dso_s),1)*(10^(par.uc_d_flat/20)-1);
  
  
  %% get par.avr amplitude spectum
  % full spectrum
  avr_s = fft(m_avr);
  avr_s = avr_s(1:floor(par.n/2));
  avr_f = linspace(0,par.avr_sr/2,floor(par.n/2));
  
  % get odd harmonics only
  avr_f = avr_f(par.avr_pn+1:par.avr_pn*2:end-par.avr_pn);
  avr_s = avr_s(par.avr_pn+1:par.avr_pn*2:end-par.avr_pn);
  
  % amplitude and phase
  avr_s = abs(avr_s);
  
  
  %% align fundamental frequencies
  avr_s = avr_s.*(dso_s(1)/avr_s(1));
  
  %% calc. freq. transfer
  a = avr_s./dso_s;
  a = 20*log10(a');
  % add MC noise (fundamental freq. gain error)
  a = a + randn(1,length(a))*par.uc_rf;
    
  %% limit freq. transfer
  id = find(avr_f>par.f_lim,1,'first');
  if(isempty(id))
      id = length(avr_f);
  end
  avr_f = avr_f(1:id);
  a = a(1:id);
    
  %% find f for -3dB
  f0 = interp1(a,avr_f,-3,'pchip');
       
  % store results
  res.avr_f = avr_f;
  res.a = a;
  res.f0 = f0;   
  
endfunction