function [outpar] = multicore_mc_chunks(myfun,inpar,N,min_ch_size,max_ch_count,sharefld,cores)   
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% This function is used to repetitively evaluate user function with 
  %% unchanged input parameters and one return variable. It is designed for 
  %% multicore Monte-Carlo evaluation.
  %% Parameter can be any type, return value of user function must be either 
  %% scalar, vector or structure of scalars and vectors.
  %% Returned variable of this function has the same type as user function 
  %% except all particular results are combined together: Scalars are combined 
  %% to vectors, vectors to 2D arrays. In both cases every row belongs to one 
  %% call of the user function. Multicore operation requires "multicore" 
  %% package installed. To eliminate insane amount of job distribution files 
  %% in shared folder this function divides total iterations count into chunks 
  %% that are evaluated each at once (at least in Windows file sharing is 
  %% extremely slow).
  %% 
  %% Parameters:
  %% myfun: user function handle
  %% inpar: user funcctio input parameter
  %% N: number of iterations
  %% min_ch_size: minimum chunk size
  %% max_ch_count: maximum allowed number of chunks
  %% sharefld: absolute share folder path
  %% cores: number of Octave processes (+1 master which only distributes and 
  %%        collects data)
  %%
  %% Version 2013.07.16 by Stanislav Maslan, s.maslan@seznam.cz
  %% http://elektronika.kvalitne.cz     
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
  % find chunk size
  if(N<min_ch_size)
    min_ch_size = N;
  end
  if(N/min_ch_size>max_ch_count)
    min_ch_size = floor(N/max_ch_count);
  end
      
  % find chunks count
  ch_count = ceil(N/min_ch_size);
          
  % generate cellfun jobs
  par.myfun = myfun;
  par.par = inpar;
  par.N = min_ch_size;
  % 
  incell = cell(ch_count,1);
  for k=1:ch_count      
    incell{k,1} = par;      
  end
  incell{end,1}.N = N - (ch_count-1)*min_ch_size;
  
  
  % process data
  if(cores > 1)
    %% multicore processing 
        
    % create slave Octave processes
    rpth = mfilename('fullpath');
    rpth = rpth(1:strchr(rpth,'\',1,'last')-1);
    pid=[];
    for k=1:cores
      [pin,pout,spid]=popen2('octave.exe','-q');
      pid(k)=spid;
            
      fputs(pin,['cd(''' rpth ''');']);
      fputs(pin,sprintf('\n'));
      fputs(pin,['startmulticoreslave_mod(''' sharefld ''');']);
      fputs(pin,sprintf('\n'));
      fclose(pin);
      fclose(pout);  
    end
    
    % start multicore processing (modified version of startmulticoremaster() function)
    % master only manages jobs distribution and collects results, it's not processing data
    % because the implementation is totaly f.cked up and I'm too lazy to fix it ... 
    rescell = startmulticoremaster_mod(@multicore_mc_chunks_slave,incell,sharefld,0);
    
    % kill slave Octave processes
    for k=1:cores
      if(ispc())
        % Windoze - kill by system command, pclose() not implemented yet
        syscmd=['taskkill /F /PID ' int2str(pid(k))];
        [sout,stxt] = system(syscmd);
      else
        % close pid
        pclose(pid(k));
      end
    end    
    
  else
    %% single core processing
    
    % first call to get return type
    rescell = {multicore_mc_chunks_slave(incell{1})};
    
    % allocate buffer for all jobs
    n = length(incell);
    rescell = resize(rescell,n,1);
    
    % process rest of the jobs
    kn = 100/n;
    for k = 2:n
      rescell{k} = multicore_mc_chunks_slave(incell{1});
      printf('Progress ... %d%%\r',kn*k);
    end
    printf('\n');
    
      
  end
  
  
    
  %%% combine result chunks into single cell array %%%
  if(isstruct(rescell{1}(1)))
    %% result is structure
            
    % create output struct prototype
    outstr = rescell{1}(1);
    
    % get struct field names
    rnms = fieldnames(outstr);
    rn = length(rnms);
    
    % init output struct fields 
    for n=1:rn
      fld = getfield(outstr,rnms{n});
      
      if(isscalar(fld))
        % is scalar
        outpar = resize(fld,N,1);                
      elseif(isvector(fld))
        % is vector 
        outpar = resize(fld,N,length(fld));                
      end
      
      % combine result cells into single variable
      p = 0;
      for k=1:length(rescell)
        for m=1:length(rescell{k});            
          outpar(p+m,:) = getfield(rescell{k}(m),rnms{n});        
        end
        p = p + length(rescell{k});
      end
           
      % resize if incomplete
      outpar = resize(outpar,p,columns(outpar));        
      
      % store into output struct
      outstr = setfield(outstr,rnms{n},outpar);      
    
    end
         
    % return
    outpar = outstr; 
    
  else
    %% result is variable
            
    % allocate output
    cl = rescell{1}(1);
    if(isscalar(cl))
      % is scalar
      outpar = resize(cl,N,1);
    elseif(isvector(cl))
      % is vector
      outpar = resize(cl,N,length(cl));
    end
    
    % combine result cells into single variable
    p = 0;
    for k=1:length(rescell)
      for m=1:length(rescell{k});            
        outpar(p+m,:) = rescell{k}(m);        
      end
      p = p + length(rescell{k});
    end
    
    % resize if incomplete
    outpar = resize(outpar,p,columns(outpar));     
    
  end
 
endfunction